
from tnthai.swathclone.SCDict import SwathDicTrie 
from tnthai.swathclone.SCDict import SwathSegmentAlgorithm 
from tnthai.swathclone.SCDict import SwathC 

segmenter = SwathC(SwathDicTrie, SwathSegmentAlgorithm)

def SafeSegment(text):
    return segmenter.Segment(text,"Safe")

def UnsafeSegment(text):
    return segmenter.Segment(text,"Unsafe")

def SmartSegment(text):
    return segmenter.Segment(text,"Smart")

def SafeSegmentBounding(text):
    return segmenter.Segment(text,"SafeBounding")



#!/usr/bin/env python
# -*- coding: UTF-8 -*-

import abc

class AbstractGramTrie:
    __metaclass__ = abc.ABCMeta
    @abc.abstractmethod
    def __init__(self,):
        pass

class AbstractSegmentAlgorithm:
    __metaclass__ = abc.ABCMeta
    @abc.abstractmethod
    def Segmenting(self, ):
        pass

class AbstractGramSegment:
    __metaclass__ = abc.ABCMeta
    @abc.abstractmethod
    def __init__(self, ):
        pass
    
    @abc.abstractmethod
    def Segment(toBeSegmentInput):
        pass
    
class PrototypeGramTrie(AbstractGramTrie):
    def __init__(self,):
        pass

class PrototypeSegmentAlgorithm(AbstractSegmentAlgorithm):
    def Segmenting(self, InputString, GramTrie):
        pass

class PrototypeGramSegment(AbstractGramSegment):
    def __init__(self, GramTrieClass, SegmentAlgorithmClass):
        self.GramTrie = GramTrieClass()
        self.SegmentAlgorithm = SegmentAlgorithmClass()
        assert isinstance(self.GramTrie, AbstractGramTrie)
        assert isinstance(self.SegmentAlgorithm, AbstractSegmentAlgorithm)
    
    def Segment(toBeSegmentInput):
        result = self.GramAlgorithm.Segmenting(toBeSegmentInput, self.GramTrie)
        return result

if __name__ == '__main__':
    c = PrototypeGramSegment(PrototypeGramTrie,PrototypeSegmentAlgorithm)

    

    
    


from setuptools import setup

def readme():
    with open('README.md') as f:
        return f.read()

setup(name='tnthai',
    version='1.6',
    description='tnthai for Python',
    long_description=readme(),
    url='https://gitlab.thinknet.co.th/research/swathclone',
    author='Supphawit',
    author_email='supphawit@thinknet.co.th',
    license='Thinknet',
    install_requires=[
        'datrie',
    ],
    scripts=['bin/tnthai-run','bin/tnthai-test'],
    keywords='tnthai thinknet thai wordsegment',
    packages=['tnthai'],
    package_dir={'tnthai': 'src/tnthai'},
    package_data={
        'tnthai': ['swathclone/*','*.md','swathclone/dict/*.trie','swathclone/tests/*.py']
    },
)


#!/usr/bin/env python
#  -*- coding: UTF-8 -*-

import unittest

try:
    import tnthai.segment as tn
except ImportError:
    raise ImportError("Can't import package please check site-packages in your python directory") 

# result,mode = segmenter.Segment("ทดสอบการทำงานของระบบ") 
class TestPostInstall(unittest.TestCase):
    def test_is_work_case_one(self):
        safe_one = tn.SafeSegment("ทดสอบการทำงานของระบบ")
        safe_two = tn.SafeSegment("ทดสอบกฮารทำงานของระบบ")
        unsafe = tn.UnsafeSegment("ทดสอบการทำงานของระบบ")
        smart_one = tn.SmartSegment("ทดสอบการทำงานของระบบ")
        smart_two = tn.SmartSegment("ทดสอบกฮารทำงานของระบบ")

        self.assertEqual(safe_one[0],"Safe")
        print ("\nCase 1: \nSuccess in SafeSegment with spellings")
        self.assertEqual(safe_two[1],[])
        self.assertEqual(safe_two[0],"Safe")
        print ("Success in SafeSegment with misspellings")
        self.assertEqual(unsafe[0],"Unsafe")
        print ("Success in UnsafeSegment")     
        self.assertEqual(smart_one[0],"Safe") 
        print ("Success in SmartSegment with spellings")   
        self.assertEqual(smart_two[0],"Unsafe") 
        print ("Success in SmartSegment with missspellings")

    def test_is_work_case_two(self):
        safe_one = tn.SafeSegment("คนแก่ขนของ")
        safe = safe_one[1][0][0]
        safe_two = tn.SafeSegment("คนแก่สขนของ")
        unsafe = tn.UnsafeSegment("คนแก่ขนของ")
        smart_one = tn.SmartSegment("คนแก่ขนของ")
        smart_two = tn.SmartSegment("คนแก่สขนของ")

        self.assertEqual(safe_one[0],"Safe")
        print ("\nCase 2: \nSuccess in SafeSegment with spellings")
        self.assertEqual(safe_two[1],[])
        self.assertEqual(safe_two[0],"Safe")
        print ("Success in SafeSegment with misspellings")
        self.assertEqual(unsafe[0],"Unsafe")
        print ("Success in UnsafeSegment")     
        self.assertEqual(smart_one[0],"Safe") 
        print ("Success in SmartSegment with spellings")   
        self.assertEqual(smart_two[0],"Unsafe") 
        print ("Success in SmartSegment with missspellings") 

if __name__ == '__main__':
    unittest.main()


#!/usr/bin/env python
#  -*- coding: UTF-8 -*-

from tnthai.swathclone import AbstractWordSegment as AWS
import os, sys
import datrie
import string
# import cProfile
# import Timecount
import json

sys.setrecursionlimit(1000000)

class SwathDicTrie(AWS.PrototypeGramTrie):
    def __init__(self, ):
        # Initialize GramTrie
        self.Trie = datrie.Trie.load(os.path.dirname(os.path.abspath(__file__))+'/dict/my.trie')

# SegmentAlgorithm class
class SwathSegmentAlgorithm(AWS.PrototypeSegmentAlgorithm):
    def __init__(self, ):
        self.globalResult = list()
        self.segmentResult = list()
        self.failSegmentRemainText = []
        self.positive_backtrack = {}
        self.negative_backtrack = {}
        self.setResult = set()
        
    def Segmenting(self, InputString, GramTrie, Mode):
        # check python version 
        if (sys.version_info > (3, 0)):
            self.inputWord = InputString
            # Check mode 
            if (Mode == "Safe"):
                # clear variables                 
                self.segmentResult = list()
                # call recursive Segment_safe to find only first result
                self.Segment_safe(self.inputWord, GramTrie)
                # clone result from temp
                result = list(self.globalResult)
                # clear variables 
                self.globalResult = list()
                self.segmentResult = list()
                return Mode,result

            elif (Mode == "SafeBounding"):
                # clear variables 
                # do safe bounding segment
                self.Segment_safe_with_bounding(0,'',self.inputWord, GramTrie)
                # clone result from temp
                result = self.setResult
                # clear variables 
                self.positive_backtrack = {}
                self.negative_backtrack = {}
                self.setResult = set()
                return  Mode,result

            elif (Mode == "Unsafe"):
                # do unsafe segment
                self.Segment_unsafe(self.inputWord, GramTrie)
                # clone result from temp
                result = list(self.globalResult)
                # clear global Temp result
                self.globalResult = list() 
                return Mode,result

            elif (Mode == "Smart"):
                Mode = "Safe"
                # call recursive Segment_safe to find only first result
                self.Segment_safe(self.inputWord, GramTrie)
                # clone result from temp
                result = list(self.globalResult)
                # empty blacklist
                self.failSegmentRemainText = []
                #  if no result in safe
                if len(result) == 0:
                    Mode = "Unsafe"
                    # do unsafe segment
                    self.Segment_unsafe(self.inputWord, GramTrie)
                    # clone result from temp
                    result = list(self.globalResult)
                
                # clear global Temp result
                self.globalResult = list()
                
                return Mode,result
        else:
            # This is where python2 going to work
            
            # python2 have to decode thai word before use
            self.inputWord = InputString.decode('utf-8')
            # Check mode 
            if (Mode == "Safe"):
                # clear variables                 
                self.segmentResult = list()
                # call recursive Segment_safe to find only first result
                self.Segment_safe(self.inputWord, GramTrie)
                # clone result from temp
                result = list(self.globalResult)
                # clear variables 
                self.globalResult = list()
                self.segmentResult = list()
                return Mode,result

            elif (Mode == "SafeBounding"):
                # clear variables 
                # do safe bounding segment
                self.Segment_safe_with_bounding(0,'',self.inputWord, GramTrie)
                # clone result from temp
                result = self.setResult
                # clear variables 
                self.positive_backtrack = {}
                self.negative_backtrack = {}
                self.setResult = set()
                return  Mode,result

            elif (Mode == "Unsafe"):
                # do unsafe segment
                self.Segment_unsafe(self.inputWord, GramTrie)
                # clone result from temp
                result = list(self.globalResult)
                # clear global Temp result
                self.globalResult = list() 
                return Mode,result

            elif (Mode == "Smart"):
                Mode = "Safe"
                # call recursive Segment_safe to find only first result
                self.Segment_safe(self.inputWord, GramTrie)
                # clone result from temp
                result = list(self.globalResult)
                # empty blacklist
                self.failSegmentRemainText = []
                #  if no result in safe
                if len(result) == 0:
                    Mode = "Unsafe"
                    # do unsafe segment
                    self.Segment_unsafe(self.inputWord, GramTrie)
                    # clone result from temp
                    result = list(self.globalResult)
                
                # clear global Temp result
                self.globalResult = list()
                
                return Mode,result
    
    def Segment_safe(self, remainText, trie):
        
        # Bug Check Accurate Found by Army 27 June 
        # if remainText in cache fail so not trying segment
        # if remainText in self.failSegmentRemainText:
        #    return False
        
        # this always be true
        assert ("".join(self.segmentResult) + remainText) == self.inputWord
        
        if len(remainText) > 0:
            FirstChar = remainText[0]
            if self.isLeadingChar(FirstChar):
                Prefixes = trie.prefixes(remainText)
                resultOrCount = False
                
                for i in range(len(Prefixes)-1,-1,-1):
                    selectedPrefix = Prefixes[i]
                    self.segmentResult.append(selectedPrefix)
                    suffixRemainText = remainText[len(selectedPrefix):]
                    prefixResult = self.Segment_safe(suffixRemainText, trie)
                    self.segmentResult.pop()
                    resultOrCount = resultOrCount or prefixResult
                
                if resultOrCount == False:
                    #self.failSegmentRemainText.append(remainText)
                    pass
                
            else:
                if FirstChar in string.ascii_letters+string.digits+string.printable:

                    for i in string.punctuation,string.whitespace,string.ascii_letters,string.digits:
                        if FirstChar in i:
                            remain,othersCharResult = self.isOtherChar(FirstChar,remainText,i)

                    self.segmentResult.append(othersCharResult)
                    self.Segment_safe(remain, trie)
                    self.segmentResult.pop()

        else:
            # all remainText consume so. get the segmenting result
            self.globalResult.append(list(self.segmentResult))

    def Segment_safe_with_bounding(self, curID, select, remainText, trie):
        
        # same bug as Segment_safe
        # if remainText in self.failSegmentRemainText:
        #     return False

        # this always be true
        assert ("".join(self.segmentResult) + remainText) == self.inputWord
        
        if self.checkPBI(curID):
            return

        if self.checkNBI(curID):
            return  

        if len(remainText) > 0:
            FirstChar = remainText[0]
            if self.isLeadingChar(FirstChar):
                Prefixes = trie.prefixes(remainText)
                resultOrCount = False               

                for i in range(len(Prefixes)-1,-1,-1):
                    selectedPrefix = Prefixes[i]
                    curID += len(selectedPrefix)
                    self.segmentResult.append(selectedPrefix)
                    suffixRemainText = remainText[len(selectedPrefix):]
                    prefixResult = self.Segment_safe_with_bounding(curID,selectedPrefix, suffixRemainText, trie)
                    self.segmentResult.pop()
                    curID -= len(selectedPrefix)
                    resultOrCount = resultOrCount or prefixResult
                
                if resultOrCount == False:
                    self.negative_backtrack[curID-len(select)] = True   
                
            else:
                if FirstChar in string.ascii_letters+string.digits+string.printable:

                    for i in string.punctuation,string.whitespace,string.ascii_letters,string.digits:
                        if FirstChar in i:
                            remain,othersCharResult = self.isOtherChar(FirstChar,remainText,i)

                    self.segmentResult.append(othersCharResult)
                    self.Segment_safe_with_bounding(curID+len(othersCharResult),'',remain, trie)
                    self.segmentResult.pop()
                        
                else:
                    # in case other langauge , skip letter by letter
                    self.segmentResult.append(remainText[0])
                    self.Segment_safe_with_bounding(curID+1,select,remainText[1:], trie)
                    self.segmentResult.pop()
                    
        else:
            # all remainText consume so. get the segmenting result
            self.PBI()

    def Segment_unsafe(self, remainText, trie):
        # To get only first result
        if len(self.globalResult) > 0:
            return
    
        # this always be true
        assert ("".join(self.segmentResult) + remainText) == self.inputWord
        
        if len(remainText) > 0:
            FirstChar = remainText[0]
            if self.isLeadingChar(FirstChar):
                Prefixes = trie.prefixes(remainText)
                for i in range(len(Prefixes)-1,-1,-1):
                    self.segmentResult.append(Prefixes[i])
                    self.Segment_unsafe(remainText[len(Prefixes[i]):], trie)
                    self.segmentResult.pop()
                j=0
                unsafePrefixes=""
                while j<len(remainText):
                    Prefixes = trie.prefixes(remainText[j:])
                    j=j+1
                    if len(Prefixes)>0 :
                        unsafePrefixes=remainText[:j-1]
                        break
                    else:
                        unsafePrefixes=remainText[:j]
                    if j==len(remainText):
                        unsafePrefixes = remainText
                        break
                    if not self.isLeadingChar(remainText[j]):
                        unsafePrefixes = remainText[:j]
                        break
                self.segmentResult.append(unsafePrefixes)
                self.Segment_unsafe(remainText[len(unsafePrefixes):], trie)
                self.segmentResult.pop()
            else:
                if FirstChar in string.ascii_letters+string.digits+string.printable:
                    
                    for i in string.punctuation,string.whitespace,string.ascii_letters,string.digits:
                        if FirstChar in i:
                            remain,othersCharResult = self.isOtherChar(FirstChar,remainText,i)

                    self.segmentResult.append(othersCharResult)
                    self.Segment_safe(remain, trie)
                    self.segmentResult.pop()
                
                else:
                    self.segmentResult.append(remainText[0])
                    self.Segment_unsafe(remainText[1:], trie)
                    self.segmentResult.pop()
        else :
            self.globalResult.append(list(self.segmentResult))
                    
    def isLeadingChar(self, ch):
        # 0E4E 0e4F 0e5a 0e5b
        thai_unicode_char = u'กขฃคฅฆงจฉชซฌญฎฏฐฑฒณดตถทธนบปผฝพฟภมยรฤลฦวศษสหฬอฮฯะ ัาำ ิ ี ึ ื ุ ู ฺ ฿เแโใไๅๆ ็ ่ ้ ๊ ๋ ์ ํ ๐๑๒๓๔๕๖๗๘๙' + u"\u0e4e\u0e4f\u0e5a\u0e5b"
        return ch in thai_unicode_char.replace(" ","")
        # return ch in u'กขฃคฅฆงจฉชซฌญฎฏฐฑฒณดตถทธนบปผฝพฟภมยรลวศศษสหฬอฮะึาิูแุีเัโไใ ่ ้ ๊ ๋ ำ็ฤ์ฦํฯืๆ ฺ '.replace(" ","")
        # return ord(ch) >= ord('ก'.decode('utf-8')) and  ord(ch) <= ord('ฮ'.decode('utf-8'))
   
    def isOtherChar(self,first,remain,ch):
        if first in ch:
            j = 0
            while remain[j] in ch:
                j+=1
                if len(remain) == j:
                    break
            return remain[j:], remain[:j]
   
    def PBI(self):
        prv = 0
        for i in range(0,len(self.segmentResult)):
            if prv == 0:
                self.setResult.add((prv,self.segmentResult[i]))
                prv += len(self.segmentResult[i])
            else:
                self.positive_backtrack[prv] = True
                self.setResult.add((prv,self.segmentResult[i]))
                prv += len(self.segmentResult[i])
   
    def checkPBI(self,curID):
        if (curID in self.positive_backtrack):
            self.PBI()
            return True
    
    def checkNBI(self,curID):
        if curID in self.negative_backtrack:
            return True


# Putting altogether GramTrie and SegmentAlgorithm to function as Segmentation
class SwathC(AWS.PrototypeGramSegment):
    def __init__(self, GramTrieClass, SegmentAlgorithmClass):
        self.GramTrie = GramTrieClass()
        self.SegmentAlgorithm = SegmentAlgorithmClass()
        # also assert if trying to create other type of class
        assert isinstance(self.GramTrie, AWS.AbstractGramTrie)
        assert isinstance(self.SegmentAlgorithm, AWS.AbstractSegmentAlgorithm)
    
    def Segment(self, toBeSegmentInput, Mode):
        # Call SegmentAlgorithm to segment by passing GramTrie
        Output, Mode = self.SegmentAlgorithm.Segmenting(toBeSegmentInput, self.GramTrie.Trie, Mode)
        return Output,Mode

if __name__ == '__main__':
    # example of calling WordSegment
    WordSegment = SwathC(SwathDicTrie,SwathSegmentAlgorithm)
   
    text = sys.argv[1]
    result = WordSegment.Segment(text,"SafeBounding")

    if (len(sys.argv)-1) != 2:       
        result = WordSegment.Segment(text,"Smart") 
    elif sys.argv[2]:
        mode = sys.argv[2]
        result = WordSegment.Segment(text,mode) 

    jsonResult = {}
    jsonResult["Mode"] = result[0]
    Solutions = []
    for segs in result[1]:
        res = []
        for seg in segs:
            res.append(seg)
        Solutions.append(res)
    jsonResult["Solutions"]=Solutions
    
    print (json.dumps(jsonResult,ensure_ascii=False))



from tnthai.swathclone.SC import SwathC
from tnthai.swathclone.SC import SwathDicTrie 
from tnthai.swathclone.SC import SwathSegmentAlgorithm 

segmenter = SwathC(SwathDicTrie, SwathSegmentAlgorithm)

def SafeSegment(text):
    return segmenter.Segment(text,"Safe")

def UnsafeSegment(text):
    return segmenter.Segment(text,"Unsafe")

def SmartSegment(text):
    return segmenter.Segment(text,"Smart")

def SafeSegmentBounding(text):
    return segmenter.Segment(text,"SafeBounding")